/******* listarchiver.c
    List archiver program. Collects incoming messages into message
    archives. Written by Gabor Paller
    This program can be used as a separate command-line
    tool that accepts incoming messages on its standard input
    but normally it is forked by nlist.
			**************/


#include	<stdio.h>
#include	<stdlib.h>
#include	<sys/types.h>
#include	<sys/timeb.h>
#include	<time.h>
#include	<fcntl.h>
#include	<unistd.h>
#include	<string.h>
#include	<ctype.h>
#include	<errno.h>
#include	<setjmp.h>
#define MAIN
#include	"config.h"
#include	"decoder.h"	/* Imports from decoder.c */
#include	"imports.h"	/* imports from other objects */

#include	<stdarg.h>
#if defined SOLARIS
#include	<sys/varargs.h>
#endif
 
#define	FILEPATH	256	/* Length of the maximal file path */
#define	ENDCH				0

#define CANNOT_CHANGE_TO_DATADIR	10
#define CANNOT_OPEN_SPOOL		11
#define CANNOT_WRITE_INDEX		12
#define CANNOT_WRITE_LETTER		13
#define CANNOT_OPEN_INDEX		14
#define CANNOT_OPEN_CONFIG		15
#define	NO_LISTS			16
#define CANNOT_TRUNC_SPOOL		17
#define	CANNOT_OPEN_SCRATCH		18
#define	CANNOT_OPEN_STATUS		19
#define CANNOT_LOCK_STATUS		20

/* Jump buffer for error handling */
jmp_buf	finish;

/* Actual year */
int	ActYear;
/* Actual month */
int	ActMonth;
/* Actual message counter */
long	MessageCounter;
/* Character set of from */
int	from_chset;
/* Sender of the actual letter */
char	fromfield[256];
/* Character set of the subject */
int	subject_chset;
/* Subject of the actual letter */
char	subjfield[256];
/* Date field of the actual letter */
char	datefield[256];
/* HTML conversion string */
char	htmlstr[512];
/* Log file name */
char	logfilename[FILEPATH];
/* Name of the actually created HTML file */
char	ActFileName[FILEPATH];
/* Handle of the status file - global so that it can remain locked */
FILE	*status = NULL;
/* Date in short format as produced by letter processor */
char	shortdate[256];

/* End-of-line flag */
int	eoln;
/* Actual string to be processed */
char	actline[1024];
/* Pointer in the actual string */
char	*cptr;
/* TRUE if the message is a multipart/mime letter */
bool 	isMultiPart;
/* Boundary string if multipart is enabled */
char	boundary[256];
/* name of the month index file */
char	monthindex_name[FILEPATH];
/* temporary string */
static char	tempstr[512];

/* garble flag */
int	garble;

/* Structure for storing message parts. Content-type is followed by
   a buffer for message part */
#define MSG_PART struct msg_part
struct msg_part {
  char	content_type[32];		/* Content-type of the buffer */
  char	charset[32];			/* Character set of the buffer */
  char	encoding[32];			/* Character encoding of the buffer */
  char	disposition[256];		/* Name of the attachment (if known) */
  int	buflen;				/* Length of data in buffer */
  char	*buffer;			/* The text part itself */
};

#define MAX_MSG_PART	10		/* The maximum number of MIME parts a message may have */

/* Descriptor array storing up to MAX_MSG_PART message parts */
MSG_PART	msg_parts[MAX_MSG_PART];
/* Counter in the msg_parts table */
int		partCtr;

/* Reads a character from the line. eoln is set to TRUE if end of line
   reached */
char gch() {
  char c = ENDCH;

  eoln = FALSE;
  if( *cptr == ENDCH )
	eoln = TRUE;
  else
	c = *( cptr++ );
  return c;
}

/* Reads in a line from a specified stream using read(). Buffers the
   result and gives back a pointer to the line separated by newline
   or end-of-file. Returns NULL on error */
char *read_line( char *buffer, int maxlen,int fd ) {
  static int	sterr = 0;	/* Stored error */
  char	*p,*p1;
  int	n;

  if( sterr )			/* Error was stored in the previous call */
	return NULL;
  p = buffer;
  errno = 0;
  do {
    n = read( fd,p,1 );
    p1 = p;
    if( n > 0 ) {
      p += n;
      maxlen -= n;
      if( ( *p1 == '\n' ) || ( maxlen <= 0 ) ) {
        *p = 0;
	return buffer;
      }
    }
  } while( n > 0 );
  *p = 0;
  sterr = 1;
  return p == buffer ? NULL : buffer;
}

/* Skips the next white characters in line. eoln is set to TRUE if end of
   line */
void skipspc() {
 while( isspace( gch() ) && !eoln );
 if( !eoln )
	--cptr;
}

/* Fits a string to the actual line. If it fits, pushes the actual pointer
   after the end of the string found and returns TRUE else returns FALSE
   and leaves the string pointer in place */
int fit( char str[] ) {
  register char *t,*s,c;
  int r = FALSE;

  s = str;
  skipspc();
  if( !eoln ) {
    t = cptr;
    while( ( ( c = toupper( *s ) ) != ENDCH ) && ( c == toupper( *t ) ) ) {
      s++; t++;
    }
    if( c == ENDCH ) {
      cptr = t;
      r = TRUE;
    }
  }
  return r;
}

/* Escapes a string into CGI-coded form. Gets the string, returns a pointer
  to the result */
char *Escape( char *s ) {
  static char	result[1024];
  char	c,*dest;

  dest = result;
  while( c = *(s++) ) {
    if( ( c != ' ' ) && !isalnum( c ) ) {
      sprintf( dest,"%%%02X",c );
      dest += 3;
    } else {
      if( c == ' ' )
	c = '+';
      *(dest++) = c;
    }
  }
  *dest = 0;
  return result;
}

/* Logs the error text into the log file */
void LA_Log( char format[], ... ) {
  va_list	ap;
  FILE		*logfile;
  struct timeb	ts;
  struct tm	*tp;
  char	timestr[32];
  
  ftime( &ts );
  tp = localtime( &(ts.time) );
  strcpy( timestr,asctime( tp ) );
  timestr[ strlen( timestr ) - 1 ] = 0;
  if( ( logfile = fopen( "errorlog","a" ) ) == NULL )
	return;		/* What can we do ? Can't open error log ... */
  fprintf( logfile,"%s : ",timestr );
  va_start( ap,format );
  vfprintf( logfile,format,ap );
  va_end( ap );
  fprintf( logfile,"\n" );
  fclose( logfile );
}

/* Searches for the end of the file */
void SearchListEnd( FILE *indexfile ) {
  long	fp;

  while( !feof( indexfile ) ) {
    fp = ftell( indexfile );
    if( fgets( actline,256,indexfile ) != NULL ) {
      eoln = FALSE;
      cptr = actline;
      if( fit( "<!-- LISTEND -->" ) ) {
	fseek( indexfile,fp,SEEK_SET );
	break;
      }	/* if( fit ... ) */
    }		/* if( fgets ... ) */
  }		/* while( !feof() ) */
}

/* Reads the date and compares it with the status file's date.
   If the date changed compared to the status file, the message
   counter is set back to 0. Writes the new status file. Creates
   status file if it didn't exist. *DANGER* it keeps locked the
   status file. */
void AdjustConfigFile() {
  struct timeb ts;
  struct tm *tp;
  char	work[128];
  LOCK_STRUCT	lt;
  int	i,err;

  ActYear = 0;
  ActMonth = 0;
  MessageCounter = 0L;
l1:
  if( ( status = fopen( "status","r+" ) ) == NULL ) {
    if( ( status = fopen( "status","w" ) ) == NULL ) {
      LA_Log( "Cannot create status file" );
      longjmp( finish, CANNOT_OPEN_STATUS );
    }
    goto l1;
  }
  lt.l_type = F_WRLCK;
  lt.l_whence = 0;
  lt.l_start = 0;
  lt.l_len = 0;
/* We try ten times to lock the file */
  for( i = 0 ; i < 10 ; ++i ) {
    err = fcntl( fileno( status ), F_SETLK, &lt );
    if( err >= 0 )
      break;
    if( ( err < 0 ) && ( errno == EAGAIN ) )
        sleep( 1 );
  }
  if( err < 0 ) {
    LA_Log( "Cannot lock status file for 10 seconds" );
    fclose( status );
    longjmp( finish,CANNOT_LOCK_STATUS );
  }
  if( fgets( work,127,status ) != NULL )
	sscanf( work,"%i",&ActYear );
  if( fgets( work,127,status ) != NULL )
	sscanf( work,"%i",&ActMonth );
  if( fgets( work,127,status ) != NULL )
	sscanf( work,"%li",&MessageCounter );
          
  ftime( &ts );
  tp = localtime( &(ts.time) );
  if( ( ActYear != tp->tm_year ) || ( ActMonth != ( tp->tm_mon + 1 ) ) )
  {
    ActYear = tp->tm_year;
    ActMonth = tp->tm_mon + 1;
    MessageCounter = 0L;
  }
  fseek( status,0L,SEEK_SET );
  fprintf( status,"%i\n",ActYear );
  fprintf( status,"%i\n",ActMonth );
  fprintf( status,"%li\n",MessageCounter+1 );
/* Status file is updated but not closed and unlocked. That we will do
   at the end of the program so that other instances of this program
   keep waiting at the beginning of this routine and they don't blast
   the HTML-updating tasks */
  fflush( status );
}

/* Writes the string in HTML format, converts special characters into
   escaped version. Gets the string, returns the converted string */
char *WriteHTMLString( char actline[] )
{
  char	*p,*dp;
  char	tmp[10];
  int	cc = 0;

  dp = htmlstr;
  for( p = actline; ( *p != ENDCH ) && ( cc < 506 ) ; ++p )
    if( *p == '<' ) {
      *dp = 0;
      strcat( dp,"&lt;" );
      dp += 4;
      cc += 4;
    }
    else if( *p == '>' ) {
      *dp = 0;
      strcat( dp,"&gt;" );
      dp += 4;
      cc += 4;
    }
    else if( *p == '&' ) {
      *dp = 0;
      strcat( dp,"&amp;" );
      dp += 5;
      cc += 5;
    }
    else {
      *(dp++) = *p;
      ++cc;
    }
  *dp = 0;
  return htmlstr;
}

/* Checks the existence of the index page and creates a new index
   page if it does not exist */
void CheckIndexFile() {
  char	IndexFileName[FILEPATH];
  FILE	*indexfile,*monthindex,*subjectfile;

  sprintf( IndexFileName,"%03i%02i.html",ActYear,ActMonth );
  if( ( indexfile = fopen( IndexFileName,"r" ) ) == NULL ) {
    if( ( indexfile = fopen( IndexFileName,"w" ) ) == NULL ) {
      LA_Log( "Cannot write index file: %s",IndexFileName );
      longjmp( finish, CANNOT_WRITE_INDEX );
    }
    fprintf( indexfile,"<HTML>\n<HEAD>\n<TITLE>%s: %i %s</TITLE>\n</HEAD>\n<BODY>",
	INTL_TOC,1900+ActYear, MonthNames[ ActMonth - 1 ] );
    fprintf( indexfile,
	"<H1>%s: %i %s</H1>\n<HR>\n",
	 INTL_TOC, 1900+ActYear, MonthNames[ ActMonth - 1 ] );
    fprintf( indexfile,"<ul>\n<!-- LISTEND -->\n</ul>\n" );
    fprintf( indexfile,
      "<HR><a href=\"%s\"><img src=\"%sback.gif\"></a>\n",
			monthindex_name,ICONDIR );
    if( ( monthindex = fopen( monthindex_name,"r+" ) ) != NULL ) {
      SearchListEnd( monthindex );
      fprintf( monthindex,"<a href=\"%s\">%i %s</a><br>\n",
		IndexFileName,
		1900+ActYear,
		MonthNames[ ActMonth - 1 ] );
      fprintf( monthindex,"<!-- LISTEND -->\n" );
      fprintf( monthindex,
         "<a href=\"/\"><img src=\"%sback.gif\"></a>\n",ICONDIR );
      fclose( monthindex );
    }
  }
  fclose( indexfile );
}

/* Inserts a forward chain to the actual letter into the previous
   letter */
void InsertForwardChain()
{
  char	PrevFileName[FILEPATH],ActFileName[FILEPATH];
  FILE	*prevfile,*scratch;
  if( MessageCounter == 0L )
      return;
  sprintf( PrevFileName,"%03i%02i%li.html",ActYear,ActMonth,
		MessageCounter-1 );
  if( ( prevfile = fopen( PrevFileName,"r+" ) ) != NULL ) {
    if( ( scratch = fopen( "scratch","w" ) ) == NULL ) {
      LA_Log( "Cannot open scratch for forward chain inserting" );
      longjmp( finish, CANNOT_OPEN_SCRATCH );
    }
    while( fgets( actline,256,prevfile ) != NULL ) {
      eoln = FALSE;
      cptr = actline;
      if( fit( "<!-- FORWARDICON -->" ) ) {
        fprintf( scratch,
"<td><a href=\"%03i%02i%li.html\"><img src=\"%sforward.gif\"></a></td>\n",
	    ActYear,ActMonth,MessageCounter,ICONDIR );
      }	else /* if( fit ... ) */
          fprintf( scratch,"%s",actline );
    }		/* while( fgets ) */
    fclose( prevfile );
    fclose( scratch );
    rename( "scratch",PrevFileName );
  }		/* if( ( prevfile = ... */
}		/* void InsertForwardChain() */

/* Inserts the arrowbar at the end of the letter. Gets the file handle */
void InsertArrowBar( FILE *prevfile ) {
  fprintf( prevfile,"<center>\n<table>\n<tr>\n" );
  if( MessageCounter > 0L ) {
    fprintf( prevfile,
    "<td><a href=\"%03i%02i%li.html\"><img src=\"%sback.gif\"></a></td>\n",
	    ActYear,ActMonth,MessageCounter-1,ICONDIR );
  }
  fprintf( prevfile,
    "<td><a href=\"%03i%02i.html\"><img src=\"%sup.gif\"></a></td>\n",
	    ActYear,ActMonth,ICONDIR );
  fprintf( prevfile,"<!-- FORWARDICON -->\n</tr>\n</table>\n</center>\n" );
}

/* Writes the message part into the output file. Gets the part number and
   the output file descriptor */
void WriteMessagePart( FILE *f, int part ) {
  char *buffer;
  int buflen,i;
  bool eolf = TRUE;
  
  buflen = msg_parts[ part ].buflen;
  buffer = (char *)( msg_parts[ part ].buffer );
  for( i = 0 ; i < buflen ; ++i ) {
    if( *buffer == '<' ) {
    	fputs( "&lt;",f );
    	eolf = FALSE;
    }
    else
    if( *buffer == '>' ) {
    	fputs( "&gt;",f );
    	eolf = FALSE;
    }
    else
    if( *buffer == '&' ) {
    	fputs( "&amp;",f );
    	eolf = FALSE;
    }
    else
    if( *buffer == '\n' ) {
    	fputs( "\n",f );
    	eolf = TRUE;
    }
    else {
    	fputc( (((int)*buffer) & 0xFF),f );
    	eolf = FALSE;
    }
    ++buffer;
  }
  if( !eolf )
  	fputs( "<br>\n", f );
}

/* Writes the attachment into an attachment file and writes a hyperlink
   pointing to the attachment into the message file. Gets the name
   of the message file, file descriptor of the message file, number of the message
   part and number of the attachment to process */
void WriteAttachment( char MsgFileName[], FILE *MsgFile, int partno, int attno ) {
  char AttFileName[FILEPATH];
  FILE *attfile;
  int i;
  unsigned char *p;
  
  sprintf( AttFileName,"%s_%i",MsgFileName,attno );
  if( *( msg_parts[partno].disposition ) != 0 ) {  /* There was an attachment name */
    strcat( AttFileName,"_" );
    strcat( AttFileName,msg_parts[partno].disposition );
  } else {
/* If no attachment file name is given, try to create some meaningful file
   name from the attachment MIME type */
    if( strcasecmp( msg_parts[partno].content_type,"text/html" ) == 0 )
    	strcat( AttFileName,".html" );
  }
  if( ( attfile = fopen( AttFileName,"wb" ) ) == NULL ) {
    LA_Log( "Cannot write: %s",AttFileName );
    return;
  }
  p = (unsigned char *)msg_parts[partno].buffer;
  for( i = 0 ; i < msg_parts[partno].buflen ; ++i )
  	fputc( (int)(*p++),attfile );
  fclose( attfile );
  fprintf( MsgFile,"<a href=\"%s\">",AttFileName );
  if( *msg_parts[partno].disposition == 0 )
  	fprintf( MsgFile,INTL_ATTACHMENT,attno,msg_parts[partno].buflen );
  else
  	fprintf( MsgFile,INTL_ATTACHMENT_NAME,attno,msg_parts[partno].disposition,
  			msg_parts[partno].buflen );
  fprintf( MsgFile,"</a><br>\n" );
}

/* Reads a letter from the file stream and converts it into HTML
   format. Writes the converted letter to disk and adds a reference
   to the index file. */
void ConvertLetterToHTML() {
  FILE	*actfile;
  char	ActFileNameShort[FILEPATH];
  char	*sp,*ep;
  int	i,n;
  int	attctr = 1;
  
  CheckIndexFile();
  sprintf( ActFileNameShort,"%03i%02i%li",ActYear,ActMonth,
		MessageCounter );
  sprintf( ActFileName,"%s.html",ActFileNameShort );
  if( ( actfile = fopen( ActFileName,"w" ) ) == NULL ) {
    LA_Log( "Cannot write: %s",ActFileName );
    longjmp( finish, CANNOT_WRITE_LETTER );
  }
  InsertForwardChain();
  fprintf( actfile,"<HTML>\n<HEAD>\n<TITLE>%s</TITLE>\n</HEAD>\n<BODY>\n",
	subjfield );
/* Cutting down leading spaces from datefield */
  sp = datefield;
  while( ( *sp == ' ' ) || ( *sp == '\t' ) )
	++sp;
  strcpy( datefield,sp );
  Trim( datefield );
  Trim( fromfield );
  Trim( subjfield );
  if( *subjfield == 0 )
	strcpy( subjfield,"<no subject>" );
  from_chset = decode_quoted_fragments( fromfield );
  transfer_charset( fromfield, tempstr, from_chset, CHARSET_ISO_8859_1 );
  if( garble )
  	garble_email( tempstr );
  fprintf( actfile,"<!-- %s -->\n",WriteHTMLString( tempstr ) );
  fprintf( actfile,"<I>%s</I><P>\n",WriteHTMLString( datefield ) );
  fprintf( actfile,"<H3>%s</H3><P>\n",WriteHTMLString( tempstr ) );
  subject_chset = decode_quoted_fragments( subjfield );
  transfer_charset( subjfield, tempstr, subject_chset, CHARSET_ISO_8859_1 );
  fprintf( actfile,"<H1>%s</H1><P>\n<HR>\n",WriteHTMLString( tempstr ) );
  fprintf( actfile,"<PRE>\n" );
  for( i = 0 ; i < partCtr ; ++i ) {
/* If text/plain type and not an attachment, emit as main message
   part else write an attachment file */
    if( ( strcmp( msg_parts[i].content_type,"text/plain" ) == 0 ) &&
        ( *msg_parts[i].disposition == 0 ) )
    	WriteMessagePart( actfile,i );
    else
    	WriteAttachment( ActFileNameShort, actfile,i,attctr++ );
  }
  fprintf( actfile,"</PRE>\n" );
  InsertArrowBar( actfile );
  fclose( actfile );
/* Then the date is preceded by day abbrev - cut it */
  if( datefield[3] == ',' )
	strcpy( shortdate,datefield+5 );
  else
	strcpy( shortdate,datefield );
  if( *shortdate == '0' )
	strcpy( shortdate,shortdate+1 );
  sp = shortdate;
/* Skip four fields to get the short date */
  for( n = 0 ; ( n < 4 ) && ( sp != NULL ) ; ++n ) {
    sp = strchr( sp,' ' );
    if( sp != NULL )
	++sp;
  }
  if( sp != NULL )
	*sp = 0;
}

/* Processes the incoming mail arriving from stdin */
void ProcessIncomingMail() {
  int	i,len,linelen,buflen,obuflen;
  char	*p,*p1,*sp,*dp,IndexFileName[FILEPATH],cb[256];
  FILE	*index1,*index2;
  bool  inContent = FALSE;
  bool	searchBoundary = FALSE;
  bool  boundaryHeaders = FALSE;

  *fromfield = 0;
  *datefield = 0;
  *subjfield = 0;
  isMultiPart = FALSE;
  partCtr = 0;
  strcpy( boundary,"" );
  for( i = 0 ; i < MAX_MSG_PART ; ++i ) {
    strcpy( msg_parts[i].content_type,"" );
    strcpy( msg_parts[i].charset,"iso-8859-1" );
    strcpy( msg_parts[i].encoding,"" );
    strcpy( msg_parts[i].disposition,"" );
    msg_parts[i].buflen = 0;
    msg_parts[i].buffer = NULL;
  }
  while( read_line( actline,255,0 ) != NULL ) {
/* If we are already processing the message body */
    if( inContent ) {
/* Boundary limit? */
      if( ( *boundary != 0 ) && 
          ( strstr( actline,boundary ) != NULL ) ) {
        if( searchBoundary )
        	partCtr = 0;
        else
		++partCtr;
/* Stop reading the file if there are too many parts */
        if( partCtr >= MAX_MSG_PART )
        	goto end_read_content;
        searchBoundary = FALSE;
/* Read in headers for the new part */
	do {
	  strcpy( actline,"" );
	  if( read_line( cb,255,0 ) == NULL )
	  	goto end_read_content;
	  if( *cb == '\n' )		/* End of part headers */
	  	break;
/* Concatenate first the header if it is composed of multiple lines */
	  do {
 	    Trim( cb );
 	    strcat( actline,cb );
	    sp = cb + strlen( cb ) - 1;
	    if( *sp != ';' )
	    	break;
	    if( read_line( cb,255,0 ) == NULL )
	    	goto end_read_content;
	  } while( TRUE );
/* Then analyse its content */
	  cptr = actline;
	  if( fit( "Content-type: " ) ) {
	      sp = strtok( actline,";" );
	      while( sp != NULL ) {
	        Trim( sp );
	        cptr = sp;
	        if( fit( "Content-type: " ) )
			strcpy( msg_parts[ partCtr ].content_type,cptr );
                else
                if( fit( "charset=" ) ) {
                  cptr += 8;
                  getWord( cptr,msg_parts[ partCtr ].charset,31,NULL );
	        }
	        sp = strtok( NULL,";" );
	      }
	  }
	  else
	  if( fit( "Content-Transfer-Encoding: " ) )
	  	strcpy( msg_parts[ partCtr ].encoding,cptr );
	  else
	  if( fit( "Content-Disposition: " ) ) {
/* Fishing out the file name of the attachment */
	    sp = strcasestr( cptr,"filename=" );
	    if( sp == NULL ) {
	      sp = strcasestr( cptr,"name=" );
	      if( sp == NULL )
	    	goto end_disp;
	      sp += 5;
	    }
	    else
	    sp += 9;
	    getWord( sp,msg_parts[ partCtr ].disposition,31,NULL );
end_disp:   
	  }	
	} while( TRUE );
      } else {
/* This is a line in the message body that is not a boundary string */
/* If we are still searching for boundary string, skip it */
        if( searchBoundary )
        	continue;
/* Actual part has no Content-Type set (maybe trailing lines after
   the last part?) Skip the line */
        if( *( msg_parts[ partCtr ].content_type ) == 0 )
        	continue;
/* Let's start supposing plain encoding */
        sp = actline;        
        linelen = strlen( sp );
        if( strcasecmp( msg_parts[ partCtr ].encoding,"quoted-printable" ) == 0 )
        	linelen = decode_quoted_printable( actline,actline );
        if( strcasecmp( msg_parts[ partCtr ].encoding,"base64" ) == 0 )
          	linelen = decode_base64( actline, (unsigned char *)actline );
        dp = msg_parts[ partCtr ].buffer;
        buflen = msg_parts[ partCtr ].buflen;
        if( dp == NULL ) {
          dp = (char *)malloc( linelen );
          buflen = linelen;
          memcpy( dp,sp,linelen );
        } else {
          obuflen = buflen;
          buflen += linelen;
          dp = (char *)realloc( dp,buflen );
          memcpy( dp+obuflen,sp,linelen );
        }
        msg_parts[ partCtr ].buflen = buflen;
        msg_parts[ partCtr ].buffer = dp;
      }
/* Else we are still processing message headers (as opposed to part headers) */
    } else {
	    cptr = actline;
	    eoln = TRUE;
	    if( fit( "From: " ) ) {
	      strncpy( fromfield, cptr,256 );
	      fromfield[255] = 0;
	    } else if( fit( "Date: " ) ) {
	      strncpy( datefield, cptr,256 );
	      datefield[255] = 0;
	    } else if( fit( "Subject: " ) ) {
	      strncpy( subjfield, cptr,256 );
	      subjfield[255] = 0;
	    } else if( fit( "Content-Transfer-Encoding: " ) ) {
	      strncpy( msg_parts[0].encoding,cptr,256 );
	      msg_parts[0].encoding[255] = 0;
	      if( ( p = strchr( msg_parts[0].encoding,'\n' ) ) != NULL )
	      	*p = 0;
	    } else if( fit( "Content-type: " ) ) {
/* Check to see if it is a multipart message and  has proper boundary indicators*/
              if( ( p = strcasestr( cptr,"charset=" ) ) != NULL ) {
                p += 8;
                getWord( p,msg_parts[0].charset,31,NULL );
              }
	      if( strcasestr( cptr,"multipart" ) != NULL &&
	          ( ( sp = strcasestr( cptr,"boundary=" ) ) != NULL ) ) {
	        sp += 9;
	        getWord( sp,boundary,128,NULL );
	      	isMultiPart = TRUE;
	      }
	    }
/* If it is an empty line, this is the end of the main message header part */
	    else if( *actline == '\n' ) {
	      inContent = TRUE;
	      if( !isMultiPart ) {
	        if( *msg_parts[0].content_type == 0 )
			strcpy( msg_parts[0].content_type,"text/plain" );
	        msg_parts[0].buffer = NULL;
	        partCtr = 0;
	      } else
	      	searchBoundary = TRUE;
	    }
    }
  }
  if( !isMultiPart )
  	partCtr = 1;
end_read_content:
  ConvertLetterToHTML();
/* Updating index file */
  sprintf( IndexFileName,"%03i%02i.html",ActYear,ActMonth );
  if( ( index1 = fopen( IndexFileName,"r" ) ) == NULL ) {
    LA_Log( "Cannot open index: %s",IndexFileName );
    longjmp( finish, CANNOT_OPEN_INDEX );
  }
  if( ( index2 = fopen( "scratch","w" ) ) == NULL ) {
    LA_Log( "Cannot open scratch" );
    longjmp( finish, CANNOT_OPEN_SCRATCH );
  }
  while( fgets( actline,256,index1 ) != NULL ) {
    fprintf( index2,"%s",actline );
    cptr = actline;
    eoln = TRUE;
    if( fit( "<!-- LISTEND -->" ) ) {
      transfer_charset( subjfield, tempstr, subject_chset, CHARSET_ISO_8859_1 );
      fprintf( index2,"<li><a href=\"%s\">%s</a> : %s",ActFileName,
			WriteHTMLString( tempstr ),shortdate );
      transfer_charset( fromfield, tempstr, from_chset, CHARSET_ISO_8859_1 );
      if( garble )
      	garble_email( tempstr );
      fprintf( index2," : %s\n",WriteHTMLString( tempstr ) );      
    }
  }
  fclose( index1 );
  fclose( index2 );
  rename( "scratch",IndexFileName );
}

/* Mail archiver main program */
int main( int argc, char *argv[] ) {
  char dir[FILEPATH],c;
  int	i,err;

  *dir = 0;
  garble = 0;
  strcpy( monthindex_name,"index.html" );
  for( i = 1 ; i < argc ; ++i ) {
    if( argv[i][0] == '-' ) {
      c = toupper( argv[i][1] );
      switch( c ) {
        case 'I':
        	if( i+1 < argc ) {
        	  ++i;
        	  strcpy( monthindex_name,argv[i] );
        	}
        	break;
        
        case 'G':
        	garble = 1;
        	break;
        
        default:
        	fprintf( stderr,"Invalid switch: %s\n",argv[i] );
        	exit( 1 );
      }
    } else
    	strcpy( dir,argv[i] );
  }
  if( *dir == 0 ) {
    fprintf( stderr,"Usage: listarchiver [-i <indexfile>] <archive directory>" );
    exit( 1 );
  }
  if( chdir( dir ) != 0 ) {
    fprintf( stderr,"Cannot change to directory %s\n",dir );
    exit( 1 );
  }
  if( ( err = setjmp( finish ) ) ) {
    if( status != NULL )
	fclose( status );	/* Status file must be unlocked */
    exit( err-1 );
  }
  AdjustConfigFile();
  ProcessIncomingMail();
/* We close now the status file so that other instances of this program
   can get in */
  fclose( status );
  return 0;
}
